angular.module('service.auth', [])

.service('AuthService', function($q, $http, API, $rootScope, USER_ROLES, Auth, $state, LOCAL_USER_KEY, LOCAL_EMAIL_KEY) {
  var isAuthenticated = false;
  var auth = firebase.auth();
  var localUser;
  var role = '';

  $http.defaults.headers.common.Authorization = API.appToken;

  Auth.$onAuthStateChanged(function(firebaseUser) {
    if(!firebaseUser) {
      var user = JSON.parse(window.localStorage.getItem(LOCAL_USER_KEY));
      if(user && user.registrant){
        $http.defaults.headers.common.Authorization = API.appToken;
        login({
          registrantid: user.registrant.registrantid, 
          authType : "registrantid"
        }).then(function(ret) {
          
        });
      }
    }
  });
 
  function loadUser() {
    var user = JSON.parse(window.localStorage.getItem(LOCAL_USER_KEY));
    if (user) storeUser(user);
  }
 
  function storeUser(user) {
    window.localStorage.setItem(LOCAL_USER_KEY, JSON.stringify(user));
    window.localStorage.setItem(LOCAL_EMAIL_KEY, user.registrant.registrantemail);
    useCredentials(user);
  }
 
  function useCredentials(user) {
    isAuthenticated = true;
    localUser = user;
    var ret;
    if(user.customtoken){
      role = USER_ROLES.user;
      $rootScope.username = user.registrant.registrantfirstname + ' ' + user.registrant.registrantlastname;
      $rootScope.$broadcast('userLoggedIn');
      $http.defaults.headers.common.Authorization = user.customtoken;
      ret = Auth.$signInWithCustomToken(user.customtoken).then(function(ret){
        auth.currentUser.updateProfile({
          displayName: user.registrant.registrantfirstname + ' ' + user.registrant.registrantlastname
        });
        if(!ret.email){
          Auth.$updateEmail(user.registrant.registrantemail).then(function(ret){
          }, function(error){
            Auth.$updateEmail('guest_' + user.registrant.registrantemail);
          });
        }
        
        // update registrant to use device token
        var deviceToken = window.localStorage.getItem('deviceToken');
        if(deviceToken){
          saveDeviceToken(deviceToken);
        }
        
      }, function(error){
        destroyUserCredentials();
        // $state.go('login');
      });
    }
    
    return ret;
  }

  function saveDeviceToken(deviceToken) {
    var dfd = $q.defer();

    $http.put(API.url + '/registrants/' + localUser.registrant.registrantid, {
      deviceToken: deviceToken
    })
    .success(function(ret) {
      dfd.resolve(ret);
    })
    .error(function(data) {
      dfd.reject(data);
    });

    return dfd.promise;
  }
 
  function destroyUserCredentials() {
    $http.defaults.headers.common.Authorization = API.appToken;
    isAuthenticated = false;
    localUser = undefined;
    Auth.$signOut();
  }
 
  var login = function(user) {
    return $q(function(resolve, reject) {
      $http.post(API.url + '/auth/firebase/login', user).then(function successCallback(ret) {
        if (ret.data.error === 'false') {
          if(ret.data.data.registrants && !ret.data.data.registrants.length){
            storeUser(ret.data.data);
          }
          resolve(ret.data.data);
        } else if(ret.data){
          reject(ret.data.messages[0]);
        } else{
          reject('Unknown Error');  
        }

      }, function errorCallback(ret) {
        if(ret.data){
            reject(ret.data.messages[0]);
        }else{
            reject('Login Server Error');          
        }
      });

    });
  };
 
  var logout = function() {
    window.localStorage.removeItem(LOCAL_USER_KEY);
    window.localStorage.removeItem(LOCAL_EMAIL_KEY);
    destroyUserCredentials();
  };
 
  var isAuthorized = function(authorizedRoles) {
    if (!angular.isArray(authorizedRoles)) {
      authorizedRoles = [authorizedRoles];
    }
    return (isAuthenticated && authorizedRoles.indexOf(role) !== -1);
  };

  loadUser();

  return {
    login: login,
    logout: logout,
    destroyUserCredentials: destroyUserCredentials,
    auth: auth,
    isAuthorized: isAuthorized,
    saveDeviceToken: saveDeviceToken,
    isAuthenticated: function() {return isAuthenticated;},
    user: function() {return localUser;},
    username: function() {return 'ed';},
    role: function() {return role;}
  };
})

.factory("Auth", ["$firebaseAuth",
  function($firebaseAuth) {
    return $firebaseAuth();
  }
])
 
.factory('AuthInterceptor', function ($rootScope, $q, AUTH_EVENTS) {
  return {
    responseError: function (ret) {
      $rootScope.$broadcast({
        401: AUTH_EVENTS.notAuthenticated,
        403: AUTH_EVENTS.notAuthorized
      }[ret.status], ret);
      return $q.reject(ret);
    }
  };
})

.config(function ($httpProvider) {
  $httpProvider.interceptors.push('AuthInterceptor');
});