angular.module('service.googleMaps', [])

.factory('GoogleMaps', function($cordovaGeolocation, $ionicLoading, $rootScope, $cordovaNetwork, ConnectivityMonitor, GOOGLE_API, $ionicPlatform, DEBUG){

  var markerCache = [];
  var map = null;
  var panorama;

  function initMap(mapOptions){
    mapOptions.center = new google.maps.LatLng(mapOptions.latitude, mapOptions.longitude);
    mapOptions.mapTypeId = google.maps.MapTypeId.ROADMAP;
    mapOptions.fullscreenControl = false;
    mapOptions.keyboardShortcuts = false;
    mapOptions.disableDefaultUI = true;
    mapOptions.streetViewControl = mapOptions.streetPano ? false : true;

    map = new google.maps.Map(document.getElementById("map"), mapOptions);

    google.maps.event.addListenerOnce(map, 'idle', function(){
      enableMap();
      var marker = new google.maps.Marker({
        map: map,
        animation: google.maps.Animation.DROP,
        position: mapOptions.center
      }); 

        panorama = map.getStreetView();
      if(mapOptions.streetPano){
        panorama.setMotionTracking(true);
        panorama.setPano(mapOptions.streetPano);
        panorama.setOptions({
          fullscreenControl: false
          ,addressControl:false
          ,disableDefaultUI:true
        });
        if(mapOptions.streetViewHeading){
          panorama.setPov({
            heading: mapOptions.streetViewHeading,
            pitch: mapOptions.streetViewPitch
          });
        }
      }
      if(DEBUG){
        google.maps.event.addListener(panorama, "pov_changed", function(e) { 
           var panoInfo   = panorama.getPov();
           var thePitch   = panoInfo['pitch'];
           var isHeading  = panoInfo['heading'];
           console.log('heading: ' + isHeading, 'pitch: ' + thePitch, 'panoID: ' + panorama.pano);
        });
      }

      if(mapOptions.infoWindow){
        var infowindow = new google.maps.InfoWindow();
        infowindow.setContent(mapOptions.infoWindow);
        infowindow.open(map, marker);
      }

    });

    $cordovaGeolocation.getCurrentPosition({timeout: 10000, enableHighAccuracy: true}).then(function(position){
      var latLng = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
    }, function(error){
    });

  }

  function enableMap(){
    $ionicLoading.hide();
  }

  function disableMap(){
    $ionicLoading.show({
      template: 'You must be connected to the Internet to view this map.'
    });
  }

  function loadGoogleMaps(mapOptions){

    $ionicLoading.show({
      template: 'Loading Google Maps'
    });

    //This function will be called once the SDK has been loaded
    window.mapInit = function(){
      initMap(mapOptions);
    };  

    //Create a script element to insert into the page
    var script = document.createElement("script");
    script.type = "text/javascript";
    script.id = "googleMaps";
    script.src = '//maps.google.com/maps/api/js?key=' + GOOGLE_API + '&callback=mapInit';
    document.body.appendChild(script);
  }

  function checkLoaded(){
    if(typeof google == "undefined" || typeof google.maps == "undefined"){
      loadGoogleMaps();
    } else {
      enableMap();
    }       
  }

  function addInfoWindow(marker, message, record) {
    var infoWindow = new google.maps.InfoWindow({
        content: message
    });

    google.maps.event.addListener(marker, 'click', function () {
        infoWindow.open(map, marker);
    });
  }

  function addConnectivityListeners(){

    if(ionic.Platform.isWebView()){
      $ionicPlatform.ready(function() {
        // Check if the map is already loaded when the user comes online, if not, load it
        $rootScope.$on('$cordovaNetwork:online', function(event, networkState){
          checkLoaded();
        });

        // Disable the map when the user goes offline
        $rootScope.$on('$cordovaNetwork:offline', function(event, networkState){
          disableMap();
        });
      });
    }
    else {

      //Same as above but for when we are not running on a device
      window.addEventListener("online", function(e) {
        checkLoaded();
      }, false);    

      window.addEventListener("offline", function(e) {
        disableMap();
      }, false);  
    }

  }

  return {
    toggleStreetView: function(){
      var toggle = panorama.getVisible();
      if (toggle === false) {
       panorama.setVisible(true);
      } else {
       panorama.setVisible(false);
      }
    },
    init: function(mapOptions){

      if(typeof google == "undefined" || typeof google.maps == "undefined"){

        console.warn("Google Maps SDK needs to be loaded");

        disableMap();

        if(ConnectivityMonitor.isOnline()){
          loadGoogleMaps(mapOptions);
        }
      }
      else {
        if(ConnectivityMonitor.isOnline()){
          initMap(mapOptions);
          enableMap();
        } else {
          disableMap();
        }
      }

      addConnectivityListeners();

    }
  };

})

;